package main

import (
	"encoding/gob"
	"fmt"
	"os"
)

const TEMP_FILE = "test.gob"

func write(e interface{}) error {
	os.Remove(TEMP_FILE)
	file, err := os.Create(TEMP_FILE)
	if err != nil {
		return err
	}
	defer file.Close()

	encoder := gob.NewEncoder(file)
	err = encoder.Encode(e)
	if err != nil {
		return err
	}

	return nil
}

func read() ([]string, error) {
	var e []string
	file, err := os.Open(TEMP_FILE)
	if err != nil {
		return nil, err
	}
	defer file.Close()

	decoder := gob.NewDecoder(file)
	err = decoder.Decode(&e)
	if err != nil {
		return nil, err
	}
	return e, nil
}

func read2() ([]City, error) {
	var c []City
	file, err := os.Open(TEMP_FILE)
	if err != nil {
		return nil, err
	}
	defer file.Close()

	decoder := gob.NewDecoder(file)
	err = decoder.Decode(&c)
	if err != nil {
		return nil, err
	}
	return c, nil
}

type Addr struct {
	Street string
}

// 命名嵌套结构体
type City struct {
	name string
	Addr
}

func main() {
	// 数组验证
	cities := []string{"shanghai", "zhengzhou", "shangqiu"}
	_ = write(cities)

	e, err := read()
	fmt.Println(e, err)

	// 结构体验证，嵌套的属性必须可访问
	cities2 := []City{
		City{
			name: "shanghai",
			Addr: Addr{Street: "huangpuqu"},
		},
		{
			name: "zhengzhou",
			Addr: Addr{Street: "zhongyuanqu"},
		},
	}
	err = write(cities2)
	fmt.Println(err)
	c2, err := read2()
	fmt.Printf("%#v %#v\n", c2, err)
}
